@@ -1,8 +1,11 @@
+import os
 import time
+import json
+import fcntl
+from uuid import uuid4
 from pathlib import Path
 from collections import defaultdict
-from json import loads, load, dump
-from uuid import uuid4
+from contextlib import contextmanager
 from typing import Dict, List, Tuple, Any, Callable, Generic, Union, cast
 from ..common import (
     T, R, L, Locator, DefaultLocator, Editor, DefaultEditor)
@@ -25,44 +28,50 @@ class JsonRepository(Repository, Generic[T]):
         self.locator = locator or DefaultLocator()
         self.editor = editor or DefaultEditor()
 
+    async def setup(self) -> None:
+        if not self.file_path.exists():
+            self.file_path.parent.mkdir(parents=True, exist_ok=True)
+            with locked_open(str(self.file_path), 'w') as f:
+                f.write("{}")
+
     async def add(self, item: Union[T, List[T]]) -> List[T]:
+        await self.setup()
 
         items = item if isinstance(item, list) else [item]
 
         data: Dict[str, Any] = defaultdict(lambda: {})
-        if self.file_path.exists():
-            data.update(loads(self.file_path.read_text()))
+        with locked_open(str(self.file_path), 'r+') as f:
+            data.update(json.loads(f.read()))
 
-        for item in items:
-            item.updated_at = int(time.time())
-            item.updated_by = self.editor.reference
-            item.created_at = item.created_at or item.updated_at
-            item.created_by = item.created_by or item.updated_by
+            for item in items:
+                item.updated_at = int(time.time())
+                item.updated_by = self.editor.reference
+                item.created_at = item.created_at or item.updated_at
+                item.created_by = item.created_by or item.updated_by
 
-            data[self.collection][item.id] = vars(item)
+                data[self.collection][item.id] = vars(item)
 
-        self.file_path.parent.mkdir(parents=True, exist_ok=True)
-        with self.file_path.open('w') as f:
-            dump(data, f, indent=2)
+            f.seek(f.truncate(0))
+            f.write(json.dumps(data, indent=2))
 
         return items
 
     async def remove(self, item: Union[T, List[T]]) -> bool:
-
-        items = item if isinstance(item, list) else [item]
         if not self.file_path.exists():
             return False
 
-        with self.file_path.open('r') as f:
-            data = load(f)
+        items = item if isinstance(item, list) else [item]
 
-        deleted = False
-        for item in items:
-            deleted_item = data[self.collection].pop(item.id, None)
-            deleted = bool(deleted_item) or deleted
+        with locked_open(str(self.file_path), 'r+') as f:
+            data = json.loads(f.read())
 
-        with self.file_path.open('w') as f:
-            dump(data, f, indent=2)
+            deleted = False
+            for item in items:
+                deleted_item = data[self.collection].pop(item.id, None)
+                deleted = bool(deleted_item) or deleted
+
+            f.seek(f.truncate(0))
+            f.write(json.dumps(data, indent=2))
 
         return deleted
 
@@ -70,19 +79,18 @@ class JsonRepository(Repository, Generic[T]):
         if not self.file_path.exists():
             return 0
 
-        self.file_path.parent.mkdir(parents=True, exist_ok=True)
+        with locked_open(str(self.file_path), 'r') as f:
+            data = json.loads(f.read())
 
-        with self.file_path.open('r') as f:
-            data = load(f)
+            count = 0
+            domain = domain or []
+            filter_function = self.filterer.parse(domain)
+            for item_dict in list(data[self.collection].values()):
+                item = self.constructor(**item_dict)
+                if filter_function(item):
+                    count += 1
 
-        count = 0
-        domain = domain or []
-        filter_function = self.filterer.parse(domain)
-        for item_dict in list(data[self.collection].values()):
-            item = self.constructor(**item_dict)
-            if filter_function(item):
-                count += 1
-        return count
+            return count
 
     async def search(self, domain: Domain,
                      limit: int = None, offset: int = None,
@@ -91,22 +99,24 @@ class JsonRepository(Repository, Generic[T]):
         if not self.file_path.exists():
             return items
 
-        with self.file_path.open('r') as f:
-            data = load(f)
+        with locked_open(str(self.file_path), 'r') as f:
+            data = json.loads(f.read())
             items_dict = data.get(self.collection, {})
 
-        filter_function = self.filterer.parse(domain)
-        for item_dict in items_dict.values():
-            item = self.constructor(**item_dict)
+            filter_function = self.filterer.parse(domain)
+            for item_dict in items_dict.values():
+                item = self.constructor(**item_dict)
 
-            if filter_function(item):
-                items.append(item)
+                if filter_function(item):
+                    items.append(item)
+
+            if offset is not None:
+                items = items[offset:]
+            if limit is not None:
+                items = items[:limit]
+            if not order:
+                return items
 
-        if offset is not None:
-            items = items[offset:]
-        if limit is not None:
-            items = items[:limit]
-        if order:
             fields = order.lower().split(',')
             for field in reversed(fields):
                 key, *direction = field.split()
@@ -114,9 +124,17 @@ class JsonRepository(Repository, Generic[T]):
                     items, key=lambda item: getattr(item, key),
                     reverse=('desc' in direction)))
 
-        return items
+            return items
 
     @property
     def file_path(self) -> Path:
         return (Path(self.data_path) / self.locator.zone /
                 self.locator.location / f"{self.collection}.json")
+
+
+@contextmanager
+def locked_open(filename, mode='r'):
+    with open(filename, mode) as file:
+        fcntl.flock(file, fcntl.LOCK_EX)
+        yield file
+        fcntl.flock(file, fcntl.LOCK_UN)
